function [clustermax,clustermax_weighted,nclusters,clustersize,clustersize_weighted,clusters] = fl_find_clusters(map)
% function [clustermax,clustermax_weighted,nclusters,clustersize,clustersize_weighted,clusters] = find_clusters(map)
%
% Finds clusters (connected components) in maps of any dimension. The clusters consist of map locations with non-zero elements. 
% Useful for cluster-based statistical procedures 
% 
% INPUTS:
%   map: statistical map of any dimension (1D, 2D, 3D, 4D, etc)
%
% OUTPUTS:
%   clustermax:     largest size of a cluster (size defined by number of voxels within a cluster)
%   clustermax_weighted:        largest size of a cluster (size defined by sum of map values for voxels within a cluster)
%                               clustermax and clustermax_weighted may be from different clusters
%   nclusters:      number of clusters
%   clustersize:    vector of sizes of all clusters (in decending order)
%   clustersize_weighted:   vector of sizes of all clusters (weighted; sum of map values within each cluster)
%                           results follow a 'clustersize' decending order
%   clusters:       cell array indicating the map locations in each cluster: clusters{i} = [map locations in single indexing scheme] 
%                   the clusters are returned in decending order based on the number of voxels within each cluster (not by cluster weights)
%
% Author: Dimitrios Pantazis, MIT, September 2016
%

%initialize
N = ndims(map); %map number of dimensions
dn = size(map); %dimensions
nvoxels = prod(size(map)); %number of voxels
mapmask = map~=0; %find non-zero voxels, convert to logical (possible NaNs exist)
ndx = find(mapmask); %index of suprathreshold voxels
ndx(isnan(map(ndx)))=[]; %remove NaNs
nsupvoxels = length(ndx); %number of suprathreshold voxels
i = 0; %no clusters initially
clusters{1} = []; %initially empty cluster

%if no suprathreshold voxels exist
if isempty(ndx)
    clustermax = 0;
    clustermax_weighted = 0;
    nclusters = 0;
    clusters = {[]};
    clustersize = 0;
    clustersize_weighted = 0;
    return
end

%template of neighbouring coordinates, used to expand a cluster (expndx)
expndx = rem(floor([0:3^N-1]' * 3.^(-N+1:0)),3) - 1; %build a list of neighbor indexes
s = sum(abs(expndx),2);
expndx(s == 0 | s == N,:) = []; %remove points with no common edge (all dimensions change) or same point
expndx = expndx';

%find clusters
for k = 1:nsupvoxels %for all supratheshold voxels

    %index of suprathreshold voxel
    svox_ndx = ndx(k); 
    
    %if voxel already belongs to a cluster, continue to next iteration 
    if ~mapmask(svox_ndx) %if map is 0
        continue
    end

    %create new cluster
    i = i+1;
    clustersize(i) = 1; %new cluster
    clustersize_weighted(i) = single(map(svox_ndx));
    clusters{i} = {svox_ndx}; %new cluster voxel
    mapmask(svox_ndx) = 0; %found this voxel
    cluster_exp = svox_ndx; %initialize cluster expand variable
    
    %expand cluster
    while ~isempty(cluster_exp) %while the cluster keeps expanding
   
        %convert cluster_exp to multiple indexing scheme (from single indexing scheme)
        v = cell(N,1);
        [v{:}] = ind2sub(size(mapmask),cluster_exp(:));
        
        %apply neighborhood template to voxels v (separately per dimension)
        for n = 1:N
            vexp{n} = bsxfun(@plus,v{n},expndx(n,:)); %apply in each dimension
            vexp{n} = vexp{n}(:); %convert to column
            ndx0{n} = find(~vexp{n})'; %indices of 0 points
            ndxout{n} = find(vexp{n} == dn(n)+1)'; %indices of points exceeding map dimension
        end
        ndx_rm = [ndx0{:} ndxout{:}]; %indices of all out of boundary voxels
        for n = 1:N
            vexp{n}(ndx_rm) = []; %remove out of boundary voxels
        end
        cluster_exp = sub2ind(size(mapmask),vexp{:}); %convert to single indexing
        cluster_exp = cluster_exp(mapmask(cluster_exp)); %remove subthreshold voxels (or already found)
        cluster_exp = unique(cluster_exp); %keep unique voxels
        clustersize(i) = clustersize(i) + numel(cluster_exp); %update cluster size
        clustersize_weighted(i) = clustersize_weighted(i) + sum(map(cluster_exp)); %update weighted cluster size (sum map elements)
        clusters{i}{end+1} = cluster_exp; %expand cluster, using multiple cells is fast
        mapmask(cluster_exp) = 0; %remove voxels from map (already found)
    
    end
end

%assemble clusters
nclusters = length(clusters);
for i = 1:length(clusters)
    clusters{i} = cat(1,clusters{i}{:});
end

%order clusters with decending order (based on number of voxels, not weighted)
[clustersize,I] = sort(clustersize,'descend');
clustermax = clustersize(1);
clusters = clusters(I);

%order weighted clusters in same order as unweighted
clustersize_weighted = clustersize_weighted(I);
clustermax_weighted = max(clustersize_weighted);



