function [StatMapPv,StatMap,PermFWER] = fl_permutation_1sample(data,nperm,significance_th,tail,stat)
% function [StatMapPv,StatMap,PermFWER] = permutation_1sample(data,nperm,significance_th,tail,stat)
%
% Performs a permutation test on 'data' for difference against 0. 
% For a difference against any mean value m, use input data - m).
%
% The data array can have any number of dimensions N in the order [observations x variable1 x variable2 x ...]
%
% Data from each observation are randomly multiplied by +-1 to create permutation samples. 
% Statistical maps are then created either by averaging across the permuted observations (stat = 'means') or by
% computing a t-statistic across the permuted observations (stat = 'tstat'). 
% Finally, the pvalues of the original data are computed based on the empirical
% distribution of each element across the permutation samples
% 
% If PermFWER output is requested, a familywise error rate correction threshold is computed based on the
% empirical distribution of the maximum statistic across the permutation samples.
%
% For more information of the permutation approach, see: Pantazis et al.
% "A comparison of random field theory and permutation methods for the 
% statistical analysis of MEG data", Neuroimage 25(2):383-394 
%
% INPUTS:
%   data:   observations x variable1 x variable2 x variable3 x ... (supports all dimemsions)
%   nperm:  number of permutations
%   significance_th:    significance threshold (alpha value)
%   tail:   string 'both', or 'right'. Default is 'right'.
%   stat:   string 'means' (default) or 'tstat', statistic to be used when averaging observations. Either means, or a t-statistic
%
% OUTPUTS:
%   StatMapPv:          Pvalue map of the data
%   StatMap:            Statistical map of the data (this is either mean across observations or t-statistic across observations, depending on 'stat')
%   PermFWER.MaxStatistic:  Vector (nResample x 1) of the maximum statistic for each permutation sample
%           .Threshold: Permutation family-wise maximum statistic threshold (in StatMap units)
%           .StatMapTh: Thresholded statistical map
%
% EXAMPLE:
%   
%   nperm = 100;
%   significance_th = 0.05;
%   tail = 'right';
%   stat = 'means';
%   data = rand(100,10); %null data, 100 observations, 10 variables
%   [StatMapPv,StatMap,PermFWER] = permutation_1sample(data,nperm,significance_th,tail,stat);
%
% Author: Dimitrios Pantazis, October 2016


%FIX ERROR FOR MANY DIMENSIONS, MAXS

if ~exist('nperm') || isempty(nperm)
    nperm = 1000;
end

if ~exist('significance_th') || isempty(significance_th)
    significance_th = 0.05;
end

if ~exist('tail') || isempty(tail)
    tail = 'both';
end

if ~exist('stat') || isempty(stat)
    stat = 'means';
elseif ~any(strcmp(stat,{'means' 'tstat'}))
    disp('The variable stat must be either ''means'' or ''tstat''');
    return
end

if ~exist('tail') || isempty(tail) %if not given
    tail = 'both'; %default
elseif ~any(strcmp(tail,{'both','right'}))
    disp('The variable tail can be either ''right'' or ''both'' (two-sided test)');
    return
end

if strcmp(tail,'right') %if one-sided, right
    fun = @deal; %function handle of deal; it simply passes input to output
elseif strcmp(tail,'both') %if two-sided (both)
    fun = @abs; %function handle of abs; useful for a two-sided test
end

if nargout == 3 %if requested output includes the permutation familywise error rate threshold
    PermFWERflag = true;
else
    PermFWERflag = false;
end

%initialize
N = ndims(data); %data is observations x variable1 x variable2 x ... (supports all dimensions)
dn = size(data);
nobservations = dn(1);
cln = repmat({':'},1,N-1); %to select the N-1 dimensions

%first permutation sample is original data
if stat == 'means'
    StatMap = mean(data,1);
elseif stat == 'tstat'
    StatMap = mean(data,1) ./ (std(data)/sqrt(nobservations));
end
StatMapfun = fun(StatMap);

%maximum of original data
if PermFWERflag 
    MaxS = zeros(nperm,1,'single');
    MaxS(1) = max(StatMapfun); %ERROR for many dimensions
end

%original sample is included in the pvalues 
StatMapPv = ones(size(StatMapfun),'single'); % statistic map of pvalues

%perform permutations
% parfor i = 2:nperm
for i = 2:nperm
    permndx = sign(randn(nobservations,1,'single'));
    data_perm = gmultiply(permndx,data); 
    if stat == 'means'
        StatMapPermfun = fun(mean(data_perm,1));
    elseif stat == 'tstat'
        StatMapPermfun = fun(mean(data_perm,1) ./ (std(data_perm)/sqrt(nobservations)));
    end
    StatMapPv = StatMapPv + (StatMapPermfun >= StatMapfun); %update pvalue
    if PermFWERflag %if requested output includes PermFWER
        MaxS(i) = max(StatMapPermfun); %maximum of permutation sample
    end
end

%Fix Pvalues
StatMapPv = StatMapPv / nperm;

%if requested output includes the permutation familywise error rate threshold
if nargout==3
    PermFWER.MaxStatistic = sort(MaxS);
    PermFWER.Threshold = PermFWER.MaxStatistic(ceil(nperm*(1-significance_th)));
    PermFWER.StatMapTh = StatMapfun > PermFWER.Threshold;
end














