function [SignificantVariables,SignificantVariables_weighted,clustersize,clustersize_weighted,map] = fl_permutation_cluster_1sample(data, nperm, cluster_th, significance_th,tail,stat)
% function [SignificantVariables,SignificantVariables_weighted,clustersize,clustersize_weighted] = fl_permutation_cluster_1sample(data, nperm, cluster_th, significance_th,tail,stat)
%
% Performs cluster-size statistical inference on 'data' for difference against 0. 
% For a difference against any mean value m, use input data - m).
%
% The data array can have any number of dimensions N in the order [observations x variable1 x variable2 x ...]
%
% Data from each observation are randomly multiplied by +-1 to create permutation samples. 
% Statistical maps are then created by either averaging across the permuted observations (stat = 'means') or 
% computing a t-statistic across the permuted observations (stat = 'tstat'). 
% All permutation statistical maps are converted to pvalues and then the cluster_th threshold (in pvalue units)
% is applied to each permutation sample to identify suprathreshold clusters. The empirical
% distribution of the size of maximum suprathreshold cluster is used to assign statistical
% significance to the clusters (also in pvalue units) of the original data.
%
% INPUT:
%   data:   observations x variable1 x variable2 x variable3 x ... (supports all dimemsions)
%   nperm:  number of permutations
%   cluster_th: cluster defining threshold (in pvalue units)
%   significance_th:    significance threshold (alpha value)
%   tail:   string 'both', or 'right'. Default is 'right'.
%   stat:   string 'means' (default) or 'tstat', statistic to be used when averaging observations. Either means, or a t-statistic
%
% OUTPUT:
%   SignificantVariables:  times with significant clusters
%   clusters:           clusters above cluster-defining threshold
%   clustersize:        size of identified culsters
%
% EXAMPLE VALUES:  
%   nperm = 1000;
%   cluster_th = 0.05;
%   significance_th = 0.05;
%   tail = 'both';
%
% Author: Dimitrios Pantazis, October 2016
%

%ADD FLAG FOR PARFOR!!!
%Make VERBOSE
%ADD option for weighted cluster!

if ~exist('nperm') || isempty(nperm)
    nperm = 1000;
end

if ~exist('cluster_th') || isempty(cluster_th)
    cluster_th = 0.05;
end

if ~exist('significance_th') || isempty(significance_th)
    significance_th = 0.05;
end

if ~exist('tail') || isempty(tail)
    tail = 'right';
end

if ~exist('stat') || isempty(stat)
    stat = 'means';
elseif ~any(strcmp(stat,{'means' 'tstat'}))
    disp('The variable stat must be either ''means'' or ''tstat''');
    return
end

if ~exist('tail') || isempty(tail) %if not given
    tail = 'both'; %default
elseif ~any(strcmp(tail,{'both','right'}))
    disp('The variable tail can be either ''right'' or ''both'' (two-sided test)');
    return
end

if strcmp(tail,'right') %if one-sided, right
    fun = @deal; %function handle of deal; it simply passes input to output
elseif strcmp(tail,'both') %if two-sided (both)
    fun = @abs; %function handle of abs; useful for a two-sided test
end

%initialize
N = ndims(data); %data is observations x variable1 x variable2 x ... (supports all dimensions)
dn = size(data);
nobservations = dn(1);
cln = repmat({':'},1,N-1); %to select the N-1 dimensions

%disp('initialized')
%create permutation samples and convert them to pvalues (perms x variable1 x variable2 x ...)
StatMapPerm = zeros([nperm dn(2:end)],'single');
%disp('perm samps created')
%perform permutations
switch stat
    case 'means'
      %  tic
        StatMapPerm(1,cln{:}) = mean(data,1); %first permutation sample is original data
        parfor i = 2:nperm %parfor does not help much here, out of memory for large data
%             if ~rem(i,100)
%                 disp(['i = ' num2str(i) ' permutations']);
%             end
            permndx = sign(randn(nobservations,1,'single'));
            data_perm = gmultiply(permndx,data);
            permsample = mean(data_perm,1);
            StatMapPerm(i,:) = permsample(:); %works even for multiple dimensions, compatible with parfor
           % disp(['performed perm ' num2str(i)]);
        end
      %  toc
    case 'tstat'
        StatMapPerm(1,cln{:}) = mean(data,1) ./ (std(data)/sqrt(nobservations)); %normalize by the std of the mean
        for i = 2:nperm %parfor does not help here
            permndx = sign(randn(nobservations,1,'single'));
            data_perm = gmultiply(permndx,data);
            permsample = mean(data_perm,1) ./ (std(data_perm)/sqrt(nobservations));
            StatMapPerm(i,:) = permsample(:); %works even for multiple dimensions, compatible with parfor
        end
end    
%disp('done with perms')

% %first permutation sample is original data
% if stat == 'means'
%     StatMapPerm(1,cln{:}) = mean(data,1);
% elseif stat == 'tstat'
%     StatMapPerm(1,cln{:}) = mean(data,1) ./ (std(data)/sqrt(nobservations)); %normalize by the std of the mean
% end
% 
% %perform permutations
% tic
% parfor i = 2:nperm %parfor is not optimal here
%     permndx = sign(randn(nobservations,1,'single'));
%     data_perm = gmultiply(permndx,data); 
%     if stat == 'means'
%         permsample = mean(data_perm,1);
%     elseif stat == 'tstat'
%         permsample = mean(data_perm,1) ./ (std(data_perm)/sqrt(nobservations));
%     end
%     StatMapPerm(i,:) = permsample(:); %works even for multiple dimensions, compatible with parfor
% end     



%tic
%convert to pvalues using a custom tiedrank function; all pvalues computed %at once; when fun == abs, it will handle two-tail tests but it will have
%a single cluster if strongly positive values abruptly become strongly negative, which is improbable for relatively smooth fields
StatMapPermPV = (nperm+1 - fl_tiedrank(fun(StatMapPerm),@min))/nperm; %min function for tied ranks is critical for correct pvalue estimation
%clear StatMapPerm;
%toc
%disp('converted to pvalues')
%convert to 1-pvalue (for maximal cluster thresholding) and threshold
% StatMapPermPV = 1 - StatMapPermPV; %convert to 1-pvalues
% StatMapPermPV(StatMapPermPV<1-cluster_th) = 0; %set pvalues>=0.05 to zero (thus < 1-0.05)

%tic
%find maximum cluster for all permutation samples
map = squeeze(1 - StatMapPermPV(1,cln{:})); %convert to 1-pvalues
map(map<=1-cluster_th) = 0; %set pvalues>=0.05 to zero (thus 1-pvalue <= 1-0.05)
[clustermax,clustermax_weighted,nclusters,clustersize,clustersize_weighted,clusters] = fl_find_clusters(map);
%disp('finding max clusters')
if length(dn)==2 %if nvariables=1
    parfor i = 2:nperm
        map = squeeze(1 - StatMapPermPV(i,:)); %convert to 1-pvalues
        map(map<=1-cluster_th) = 0; %set pvalues>=0.05 to zero (thus 1-pvalue <= 1-0.05)
        [clustermax(i),clustermax_weighted(i)] = fl_find_clusters(map);
       % disp(['finding max cluster for perm ' num2str(i)]);
    end
elseif length(dn)==3
    parfor i = 2:nperm
        map = squeeze(1 - StatMapPermPV(i,:,:)); %convert to 1-pvalues
        map(map<=1-cluster_th) = 0; %set pvalues>=0.05 to zero (thus 1-pvalue <= 1-0.05)
        [clustermax(i),clustermax_weighted(i)] = fl_find_clusters(map);
    end
elseif length(dn)==4
    parfor i = 2:nperm
        map = squeeze(1 - StatMapPermPV(i,:,:,:)); %convert to 1-pvalues
        map(map<=1-cluster_th) = 0; %set pvalues>=0.05 to zero (thus 1-pvalue <= 1-0.05)
        [clustermax(i),clustermax_weighted(i)] = fl_find_clusters(StatMapPermPV(i,:,:,:));
    end
else %if more dimensions
    parfor i = 2:nperm
        map = StatMapPermPV(i,:); %rather than the memory inefficient for parfor: map = StatMapPermPV(i,cln{:});
        map = reshape(map,dn(2:end)); %works in any dimension very fast in parfor (efficient in memory for slicing the variable)
        map(map<=1-cluster_th) = 0; %set pvalues>=0.05 to zero (thus 1-pvalue <= 1-0.05)
        [clustermax(i),clustermax_weighted(i)] = fl_find_clusters(map);
    end
end
%toc
%disp('done with finding max clusters')
%find cluster threshold
clustermax = sort(clustermax);
clustermax_weighted = sort(clustermax_weighted);
th_ndx = ceil(nperm*(1-significance_th));
th = clustermax(th_ndx);
th_weighted = clustermax_weighted(th_ndx);

%output variables
clusters_keep = clustersize > th; %which clusters to keep
clusters_weighted_keep = clustersize_weighted > th_weighted;
SignificantVariables = zeros([dn(2:N) 1]); %initialize significant variables
SignificantVariables(cat(1,clusters{clusters_keep})) = 1; %set kept clusters to 1
SignificantVariables_weighted = zeros([dn(2:N) 1]);
SignificantVariables_weighted(cat(1,clusters{clusters_weighted_keep})) = 1;

clustersize = clustersize(clusters_keep); %only clusters above threshold
clustersize_weighted = clustersize_weighted(clusters_weighted_keep);
%disp('done')




