function [r,tieadj] = fl_tiedrank(x,fun)
% function [r,tieadj] = fl_tiedrank(X,fun)
%
%
% INPUTS:
%   X: vector (or matrix) to compute tied ranks columnwise
%   fun: function to use for repeated values; @mean is same as tiedrank, @min is useful for pvalues
%
% OUTPUTS:
%   r: vector (or matrix) with rank data
%
% Author: Dimitrios Pantazis, MIT, September 2016
%         Adapted from Matwork's tiedrank.m


if isa(x,'single')
    outclass = 'single';
else
    outclass = 'double';
end

if ~exist('fun')
    fun = @mean;
end

if isvector(x)
   [r,tieadj] = tr(x,fun);
else
    % Operate on each column vector of the input (possibly > 2 dimensional)
    sz = size(x);
    ncols = sz(2:end);  % for 2x3x4, ncols will be [3 4]
    r = zeros(sz,outclass);
    tieadj = zeros(ncols,outclass);
    parfor j=1:prod(ncols)
        [r(:,j), tieadj(j)] = tr(x(:,j),fun);
    end
end

% --------------------------------
function [r,tieadj] = tr(x,fun)
%TR Local tiedrank function to compute results for one column

% Sort, then leave the NaNs (which are sorted to the end) alone
[sx, rowidx] = sort(x(:));
numNaNs = sum(isnan(x));
xLen = numel(x) - numNaNs;

% Use ranks counting from low end
ranks = [1:xLen NaN(1,numNaNs)]';

tieadj = 0;
if isa(x,'single')
   ranks = single(ranks);
   tieadj = single(tieadj);
end

% Adjust for ties.  Avoid using diff(sx) here in case there are infs.
ties = sx(1:xLen-1) >= sx(2:xLen);
tieloc = [find(ties); xLen+2]; 
maxTies = numel(tieloc);

tiecount = 1;
while (tiecount < maxTies)
    tiestart = tieloc(tiecount);
    ntied = 2;
    while(tieloc(tiecount+1) == tieloc(tiecount)+1)
        tiecount = tiecount+1;
        ntied = ntied+1;
    end

    tieadj = tieadj + ntied*(ntied-1)*(ntied+1)/2;
    
    % Compute mean of tied ranks
    ranks(tiestart:tiestart+ntied-1) = fun(ranks(tiestart:tiestart+ntied-1)); %use fun = @mean for tiedrank, @min for pvalues

    tiecount = tiecount + 1;
end

% Broadcast the ranks back out, including NaN where required.
r(rowidx) = ranks;
r = reshape(r,size(x));















