function h = plot_channel(Channel,sensor_type,channel_map,mesh,flat,plot_axis)
%function h = plot_channel(Channel,channel_map,mesh,flat,plot_axis)
%
% Plots the MEG channels. Also, plots a channel_map of measurements of the channels with or
% without a mesh. If channel_map has many timepoints, it also shows a slider
%
% INPUTS
%   Channel: Channel structure produced by BrainStorm (inside the channel
%            file after importing data into brainstorm).
%            To manually define, for the ith sensor it should contain Channel(i).Loc =
%            3x1 vector of location, and Channel(i).Type = 'MEG'.
%   channel_map:     (optional) a (nSensors x nTimes) matrix with the measurements on the sensors. 
%            If nTimes > 1, a slider is shown to browse the channel_maps
%   sensor_type either 'MEG GRAD' or 'MEG MAG'
%   mesh:    'y' (default) or 'n', plots a mesh around the sensors
%   flat:    'y' (default) or 'n', to get a flat mesh
%   plot_axis: (optional) handle of the axis where to plot
%
% OUTPUT:
%   h: handle of channel surface
%
% Author: Dimitrios Pantazis, USC, December 2008
% Shridhar edits 12/11

%get channel locations
try
    ndx = 1;
    for i=1:length(Channel) %for all channels
        if strfind(Channel(i).Type,sensor_type) & isempty(strfind(Channel(i).Type,'REF')) %if MEG
            MEGndx(ndx) = i;
            chanlocs(:,ndx) = Channel(i).Loc(:,1);
            ndx = ndx+1;
        end
    end
    if ndx == 1 %if no MEG channels found
        for i=1:size(Channel,2) %for all channels
            if strfind(Channel(i).Type,'EEG')  %if EEG
                MEGndx(ndx) = i;
                chanlocs(:,ndx) = Channel(i).Loc(:,1);
                ndx = ndx+1;
            end
        end
    end
    chanlocs = chanlocs';
catch
    disp('Channel provided is not a valid brainstorm structure.')
    disp('To manually define, for the ith sensor it should contain:')
    disp('Channel(i).Loc = [3x1 vector]')
    disp('Channel(i).Type = ''MEG''')
    disp('where i = 1:nchannels and 3x1 vector is the x,y,z location of the ith sensor')
    return
end

%get inputs
if(exist('plot_axis') & ~isempty(plot_axis))
    axes(plot_axis);
else
    figure;
end
%if channel_map does not exist or is empty, assign zeros
if(~exist('channel_map')) || isempty(channel_map)
    channel_map = zeros(size(MEGndx,2),1);
end
if(~exist('mesh','var'))
    mesh = 'y';
end
if(~exist('flat','var'))
    flat = 'y';
end



%create patch (Sylvain's code)
chanlocs_orig=chanlocs;
chanlocs(:,3)=chanlocs(:,3)-max(chanlocs(:,3));
[TH,PHI,R]=cart2sph(chanlocs(:,1),chanlocs(:,2),chanlocs(:,3));
%PHI2=zeros(size(PHI));
R2=R./cos(PHI).^.2;
[Y,X]=pol2cart(TH,R2);
%Y et X sont les coordonnes projetees dans le plan 'tangent superieur' pour chaque capteur
bord = convhull(Y,X);
ncapt = size(chanlocs,1);
[center,R] = bestfitsph(chanlocs);
coordC = chanlocs-(ones(ncapt,1)*center');
tri = convhulln(coordC./(norlig(coordC)'*ones(1,3)));
keep = find(~(ismember(tri(:,1),bord) & ismember(tri(:,2),bord)& ismember(tri(:,3),bord)));
tri = tri(keep,:);

%plot patch
if exist('flat') & strcmp(flat,'y')
    h =trisurf(tri,X,Y,zeros(size(channel_map(:,1),1),1),channel_map(:,1)); %flat
    view(180,270)
else
    h =trisurf(tri,chanlocs_orig(:,1),chanlocs_orig(:,2),chanlocs_orig(:,3),channel_map(:,1));
    view(0,90);
end
set(h,'edgecolor','none','facecolor','interp')
axis equal
axis off
axis vis3d

%show mesh
if(strcmp(mesh,'y'))
    set(h,'edgecolor','black');
    set(h,'marker','o');
    set(h,'linestyle','none')
end

set(gcf,'color','white');

%show slider
if size(channel_map,2) > 1 %if many timepoints
    
    set(gca,'position',[0.2 0.3 .7 .6])
    hFig = gcf;
    [sh, ph, eh] = sliderPanel(...
        'Parent'  , hFig, ...
        'Title'   , 'Slider Panel', ...
        'Position', [0.2 0 0.6 0.15], ...
        'Backgroundcolor', [1 1 1],...
        'Min'     , 1, ...
        'Max'     , size(channel_map,2)+0.0000000000001, ...
        'Value'   , 1, ...
        'FontName', 'Verdana', ...
        'Callback', {@sliderCallback,h,channel_map});
end

    
function sliderCallback(varargin)

%assign inputs
h = varargin{3};
channel_map = varargin{4};
sldr = varargin{1};
newval = round(get(sldr,'value'));
set(sldr,'value',newval);
set(h,'facevertexcdata',channel_map(:,newval));


